# coding: utf-8

"""
    3Di API

    3Di simulation API (latest version: 3.0)   Framework release: 1.0.16   3Di core release: 2.0.11  deployed on:  07:33AM (UTC) on September 04, 2020  # noqa: E501

    The version of the OpenAPI document: 3.0
    Contact: info@nelen-schuurmans.nl
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

import re  # noqa: F401

# python 2 and python 3 compatibility library
import six

from openapi_client.api_client import ApiClient
from openapi_client.exceptions import (  # noqa: F401
    ApiTypeError,
    ApiValueError
)


class FilesApi(object):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

    def files_delete(self, id, **kwargs):  # noqa: E501
        """files_delete  # noqa: E501

        A simple ViewSet for viewing simulations  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.files_delete(id, async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param int id: A unique integer value identifying this file. (required)
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        return self.files_delete_with_http_info(id, **kwargs)  # noqa: E501

    def files_delete_with_http_info(self, id, **kwargs):  # noqa: E501
        """files_delete  # noqa: E501

        A simple ViewSet for viewing simulations  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.files_delete_with_http_info(id, async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param int id: A unique integer value identifying this file. (required)
        :param _return_http_data_only: response data without head status code
                                       and headers
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        local_var_params = locals()

        all_params = [
            'id'
        ]
        all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout'
            ]
        )

        for key, val in six.iteritems(local_var_params['kwargs']):
            if key not in all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method files_delete" % key
                )
            local_var_params[key] = val
        del local_var_params['kwargs']
        # verify the required parameter 'id' is set
        if self.api_client.client_side_validation and ('id' not in local_var_params or  # noqa: E501
                                                        local_var_params['id'] is None):  # noqa: E501
            raise ApiValueError("Missing the required parameter `id` when calling `files_delete`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'id' in local_var_params:
            path_params['id'] = local_var_params['id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # Authentication setting
        auth_settings = ['Bearer']  # noqa: E501

        return self.api_client.call_api(
            '/files/{id}/', 'DELETE',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type=None,  # noqa: E501
            auth_settings=auth_settings,
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats=collection_formats)

    def files_destroy_file(self, id, **kwargs):  # noqa: E501
        """files_destroy_file  # noqa: E501

        A simple ViewSet for viewing simulations  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.files_destroy_file(id, async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param int id: A unique integer value identifying this file. (required)
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: File
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        return self.files_destroy_file_with_http_info(id, **kwargs)  # noqa: E501

    def files_destroy_file_with_http_info(self, id, **kwargs):  # noqa: E501
        """files_destroy_file  # noqa: E501

        A simple ViewSet for viewing simulations  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.files_destroy_file_with_http_info(id, async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param int id: A unique integer value identifying this file. (required)
        :param _return_http_data_only: response data without head status code
                                       and headers
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: tuple(File, status_code(int), headers(HTTPHeaderDict))
                 If the method is called asynchronously,
                 returns the request thread.
        """

        local_var_params = locals()

        all_params = [
            'id'
        ]
        all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout'
            ]
        )

        for key, val in six.iteritems(local_var_params['kwargs']):
            if key not in all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method files_destroy_file" % key
                )
            local_var_params[key] = val
        del local_var_params['kwargs']
        # verify the required parameter 'id' is set
        if self.api_client.client_side_validation and ('id' not in local_var_params or  # noqa: E501
                                                        local_var_params['id'] is None):  # noqa: E501
            raise ApiValueError("Missing the required parameter `id` when calling `files_destroy_file`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'id' in local_var_params:
            path_params['id'] = local_var_params['id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['Bearer']  # noqa: E501

        return self.api_client.call_api(
            '/files/{id}/destroy_file/', 'DELETE',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='File',  # noqa: E501
            auth_settings=auth_settings,
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats=collection_formats)

    def files_download(self, id, **kwargs):  # noqa: E501
        """files_download  # noqa: E501

        Endpoint for downloading files.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.files_download(id, async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param int id: A unique integer value identifying this file. (required)
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Download
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        return self.files_download_with_http_info(id, **kwargs)  # noqa: E501

    def files_download_with_http_info(self, id, **kwargs):  # noqa: E501
        """files_download  # noqa: E501

        Endpoint for downloading files.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.files_download_with_http_info(id, async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param int id: A unique integer value identifying this file. (required)
        :param _return_http_data_only: response data without head status code
                                       and headers
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: tuple(Download, status_code(int), headers(HTTPHeaderDict))
                 If the method is called asynchronously,
                 returns the request thread.
        """

        local_var_params = locals()

        all_params = [
            'id'
        ]
        all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout'
            ]
        )

        for key, val in six.iteritems(local_var_params['kwargs']):
            if key not in all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method files_download" % key
                )
            local_var_params[key] = val
        del local_var_params['kwargs']
        # verify the required parameter 'id' is set
        if self.api_client.client_side_validation and ('id' not in local_var_params or  # noqa: E501
                                                        local_var_params['id'] is None):  # noqa: E501
            raise ApiValueError("Missing the required parameter `id` when calling `files_download`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'id' in local_var_params:
            path_params['id'] = local_var_params['id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['Bearer']  # noqa: E501

        return self.api_client.call_api(
            '/files/{id}/download/', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='Download',  # noqa: E501
            auth_settings=auth_settings,
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats=collection_formats)

    def files_list(self, **kwargs):  # noqa: E501
        """files_list  # noqa: E501

        A simple ViewSet for viewing simulations  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.files_list(async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param str filename:
        :param str filename__icontains:
        :param str etag:
        :param str expiry_date:
        :param str expiry_date__lte:
        :param str expiry_date__lt:
        :param str expiry_date__gte:
        :param str expiry_date__gt:
        :param str state:
        :param str state__in: Multiple values may be separated by commas.
        :param str type:
        :param str type__in: Multiple values may be separated by commas.
        :param str meta:
        :param str simulation__status__in:
        :param str simulation__status__exclude:
        :param int limit: Number of results to return per page.
        :param int offset: The initial index from which to return the results.
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: InlineResponse2002
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        return self.files_list_with_http_info(**kwargs)  # noqa: E501

    def files_list_with_http_info(self, **kwargs):  # noqa: E501
        """files_list  # noqa: E501

        A simple ViewSet for viewing simulations  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.files_list_with_http_info(async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param str filename:
        :param str filename__icontains:
        :param str etag:
        :param str expiry_date:
        :param str expiry_date__lte:
        :param str expiry_date__lt:
        :param str expiry_date__gte:
        :param str expiry_date__gt:
        :param str state:
        :param str state__in: Multiple values may be separated by commas.
        :param str type:
        :param str type__in: Multiple values may be separated by commas.
        :param str meta:
        :param str simulation__status__in:
        :param str simulation__status__exclude:
        :param int limit: Number of results to return per page.
        :param int offset: The initial index from which to return the results.
        :param _return_http_data_only: response data without head status code
                                       and headers
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: tuple(InlineResponse2002, status_code(int), headers(HTTPHeaderDict))
                 If the method is called asynchronously,
                 returns the request thread.
        """

        local_var_params = locals()

        all_params = [
            'filename',
            'filename__icontains',
            'etag',
            'expiry_date',
            'expiry_date__lte',
            'expiry_date__lt',
            'expiry_date__gte',
            'expiry_date__gt',
            'state',
            'state__in',
            'type',
            'type__in',
            'meta',
            'simulation__status__in',
            'simulation__status__exclude',
            'limit',
            'offset'
        ]
        all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout'
            ]
        )

        for key, val in six.iteritems(local_var_params['kwargs']):
            if key not in all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method files_list" % key
                )
            local_var_params[key] = val
        del local_var_params['kwargs']

        collection_formats = {}

        path_params = {}

        query_params = []
        if 'filename' in local_var_params and local_var_params['filename'] is not None:  # noqa: E501
            query_params.append(('filename', local_var_params['filename']))  # noqa: E501
        if 'filename__icontains' in local_var_params and local_var_params['filename__icontains'] is not None:  # noqa: E501
            query_params.append(('filename__icontains', local_var_params['filename__icontains']))  # noqa: E501
        if 'etag' in local_var_params and local_var_params['etag'] is not None:  # noqa: E501
            query_params.append(('etag', local_var_params['etag']))  # noqa: E501
        if 'expiry_date' in local_var_params and local_var_params['expiry_date'] is not None:  # noqa: E501
            query_params.append(('expiry_date', local_var_params['expiry_date']))  # noqa: E501
        if 'expiry_date__lte' in local_var_params and local_var_params['expiry_date__lte'] is not None:  # noqa: E501
            query_params.append(('expiry_date__lte', local_var_params['expiry_date__lte']))  # noqa: E501
        if 'expiry_date__lt' in local_var_params and local_var_params['expiry_date__lt'] is not None:  # noqa: E501
            query_params.append(('expiry_date__lt', local_var_params['expiry_date__lt']))  # noqa: E501
        if 'expiry_date__gte' in local_var_params and local_var_params['expiry_date__gte'] is not None:  # noqa: E501
            query_params.append(('expiry_date__gte', local_var_params['expiry_date__gte']))  # noqa: E501
        if 'expiry_date__gt' in local_var_params and local_var_params['expiry_date__gt'] is not None:  # noqa: E501
            query_params.append(('expiry_date__gt', local_var_params['expiry_date__gt']))  # noqa: E501
        if 'state' in local_var_params and local_var_params['state'] is not None:  # noqa: E501
            query_params.append(('state', local_var_params['state']))  # noqa: E501
        if 'state__in' in local_var_params and local_var_params['state__in'] is not None:  # noqa: E501
            query_params.append(('state__in', local_var_params['state__in']))  # noqa: E501
        if 'type' in local_var_params and local_var_params['type'] is not None:  # noqa: E501
            query_params.append(('type', local_var_params['type']))  # noqa: E501
        if 'type__in' in local_var_params and local_var_params['type__in'] is not None:  # noqa: E501
            query_params.append(('type__in', local_var_params['type__in']))  # noqa: E501
        if 'meta' in local_var_params and local_var_params['meta'] is not None:  # noqa: E501
            query_params.append(('meta', local_var_params['meta']))  # noqa: E501
        if 'simulation__status__in' in local_var_params and local_var_params['simulation__status__in'] is not None:  # noqa: E501
            query_params.append(('simulation__status__in', local_var_params['simulation__status__in']))  # noqa: E501
        if 'simulation__status__exclude' in local_var_params and local_var_params['simulation__status__exclude'] is not None:  # noqa: E501
            query_params.append(('simulation__status__exclude', local_var_params['simulation__status__exclude']))  # noqa: E501
        if 'limit' in local_var_params and local_var_params['limit'] is not None:  # noqa: E501
            query_params.append(('limit', local_var_params['limit']))  # noqa: E501
        if 'offset' in local_var_params and local_var_params['offset'] is not None:  # noqa: E501
            query_params.append(('offset', local_var_params['offset']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['Bearer']  # noqa: E501

        return self.api_client.call_api(
            '/files/', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='InlineResponse2002',  # noqa: E501
            auth_settings=auth_settings,
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats=collection_formats)

    def files_partial_update(self, id, data, **kwargs):  # noqa: E501
        """files_partial_update  # noqa: E501

        A simple ViewSet for viewing simulations  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.files_partial_update(id, data, async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param int id: A unique integer value identifying this file. (required)
        :param File data: (required)
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: File
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        return self.files_partial_update_with_http_info(id, data, **kwargs)  # noqa: E501

    def files_partial_update_with_http_info(self, id, data, **kwargs):  # noqa: E501
        """files_partial_update  # noqa: E501

        A simple ViewSet for viewing simulations  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.files_partial_update_with_http_info(id, data, async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param int id: A unique integer value identifying this file. (required)
        :param File data: (required)
        :param _return_http_data_only: response data without head status code
                                       and headers
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: tuple(File, status_code(int), headers(HTTPHeaderDict))
                 If the method is called asynchronously,
                 returns the request thread.
        """

        local_var_params = locals()

        all_params = [
            'id',
            'data'
        ]
        all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout'
            ]
        )

        for key, val in six.iteritems(local_var_params['kwargs']):
            if key not in all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method files_partial_update" % key
                )
            local_var_params[key] = val
        del local_var_params['kwargs']
        # verify the required parameter 'id' is set
        if self.api_client.client_side_validation and ('id' not in local_var_params or  # noqa: E501
                                                        local_var_params['id'] is None):  # noqa: E501
            raise ApiValueError("Missing the required parameter `id` when calling `files_partial_update`")  # noqa: E501
        # verify the required parameter 'data' is set
        if self.api_client.client_side_validation and ('data' not in local_var_params or  # noqa: E501
                                                        local_var_params['data'] is None):  # noqa: E501
            raise ApiValueError("Missing the required parameter `data` when calling `files_partial_update`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'id' in local_var_params:
            path_params['id'] = local_var_params['id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'data' in local_var_params:
            body_params = local_var_params['data']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['Bearer']  # noqa: E501

        return self.api_client.call_api(
            '/files/{id}/', 'PATCH',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='File',  # noqa: E501
            auth_settings=auth_settings,
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats=collection_formats)

    def files_processed(self, id, data, **kwargs):  # noqa: E501
        """Endpoint to which the upload-processor can enrich file objects (and their related objects) with metadata.  # noqa: E501

        For example the geotransform of raster files.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.files_processed(id, data, async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param int id: A unique integer value identifying this file. (required)
        :param FileMeta data: (required)
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        return self.files_processed_with_http_info(id, data, **kwargs)  # noqa: E501

    def files_processed_with_http_info(self, id, data, **kwargs):  # noqa: E501
        """Endpoint to which the upload-processor can enrich file objects (and their related objects) with metadata.  # noqa: E501

        For example the geotransform of raster files.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.files_processed_with_http_info(id, data, async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param int id: A unique integer value identifying this file. (required)
        :param FileMeta data: (required)
        :param _return_http_data_only: response data without head status code
                                       and headers
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        local_var_params = locals()

        all_params = [
            'id',
            'data'
        ]
        all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout'
            ]
        )

        for key, val in six.iteritems(local_var_params['kwargs']):
            if key not in all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method files_processed" % key
                )
            local_var_params[key] = val
        del local_var_params['kwargs']
        # verify the required parameter 'id' is set
        if self.api_client.client_side_validation and ('id' not in local_var_params or  # noqa: E501
                                                        local_var_params['id'] is None):  # noqa: E501
            raise ApiValueError("Missing the required parameter `id` when calling `files_processed`")  # noqa: E501
        # verify the required parameter 'data' is set
        if self.api_client.client_side_validation and ('data' not in local_var_params or  # noqa: E501
                                                        local_var_params['data'] is None):  # noqa: E501
            raise ApiValueError("Missing the required parameter `data` when calling `files_processed`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'id' in local_var_params:
            path_params['id'] = local_var_params['id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'data' in local_var_params:
            body_params = local_var_params['data']
        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['Bearer']  # noqa: E501

        return self.api_client.call_api(
            '/files/{id}/processed/', 'PUT',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type=None,  # noqa: E501
            auth_settings=auth_settings,
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats=collection_formats)

    def files_read(self, id, **kwargs):  # noqa: E501
        """files_read  # noqa: E501

        A simple ViewSet for viewing simulations  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.files_read(id, async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param int id: A unique integer value identifying this file. (required)
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: File
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        return self.files_read_with_http_info(id, **kwargs)  # noqa: E501

    def files_read_with_http_info(self, id, **kwargs):  # noqa: E501
        """files_read  # noqa: E501

        A simple ViewSet for viewing simulations  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.files_read_with_http_info(id, async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param int id: A unique integer value identifying this file. (required)
        :param _return_http_data_only: response data without head status code
                                       and headers
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: tuple(File, status_code(int), headers(HTTPHeaderDict))
                 If the method is called asynchronously,
                 returns the request thread.
        """

        local_var_params = locals()

        all_params = [
            'id'
        ]
        all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout'
            ]
        )

        for key, val in six.iteritems(local_var_params['kwargs']):
            if key not in all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method files_read" % key
                )
            local_var_params[key] = val
        del local_var_params['kwargs']
        # verify the required parameter 'id' is set
        if self.api_client.client_side_validation and ('id' not in local_var_params or  # noqa: E501
                                                        local_var_params['id'] is None):  # noqa: E501
            raise ApiValueError("Missing the required parameter `id` when calling `files_read`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'id' in local_var_params:
            path_params['id'] = local_var_params['id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['Bearer']  # noqa: E501

        return self.api_client.call_api(
            '/files/{id}/', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='File',  # noqa: E501
            auth_settings=auth_settings,
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats=collection_formats)

    def files_update(self, id, data, **kwargs):  # noqa: E501
        """files_update  # noqa: E501

        A simple ViewSet for viewing simulations  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.files_update(id, data, async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param int id: A unique integer value identifying this file. (required)
        :param File data: (required)
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: File
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        return self.files_update_with_http_info(id, data, **kwargs)  # noqa: E501

    def files_update_with_http_info(self, id, data, **kwargs):  # noqa: E501
        """files_update  # noqa: E501

        A simple ViewSet for viewing simulations  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.files_update_with_http_info(id, data, async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param int id: A unique integer value identifying this file. (required)
        :param File data: (required)
        :param _return_http_data_only: response data without head status code
                                       and headers
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: tuple(File, status_code(int), headers(HTTPHeaderDict))
                 If the method is called asynchronously,
                 returns the request thread.
        """

        local_var_params = locals()

        all_params = [
            'id',
            'data'
        ]
        all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout'
            ]
        )

        for key, val in six.iteritems(local_var_params['kwargs']):
            if key not in all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method files_update" % key
                )
            local_var_params[key] = val
        del local_var_params['kwargs']
        # verify the required parameter 'id' is set
        if self.api_client.client_side_validation and ('id' not in local_var_params or  # noqa: E501
                                                        local_var_params['id'] is None):  # noqa: E501
            raise ApiValueError("Missing the required parameter `id` when calling `files_update`")  # noqa: E501
        # verify the required parameter 'data' is set
        if self.api_client.client_side_validation and ('data' not in local_var_params or  # noqa: E501
                                                        local_var_params['data'] is None):  # noqa: E501
            raise ApiValueError("Missing the required parameter `data` when calling `files_update`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'id' in local_var_params:
            path_params['id'] = local_var_params['id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'data' in local_var_params:
            body_params = local_var_params['data']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['Bearer']  # noqa: E501

        return self.api_client.call_api(
            '/files/{id}/', 'PUT',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='File',  # noqa: E501
            auth_settings=auth_settings,
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats=collection_formats)



from openapi_client.aio.api_client import ApiClient
from openapi_client.aio.configuration import Configuration

from threedi_api_client.threedi_api_client import refresh_api_key
from threedi_api_client.config import Config, EnvironConfig


class ThreediApiClient:
    def __new__(cls, env_file=None, config=None):
        if env_file is not None:
            user_config = Config(env_file)
        elif config is not None:
            user_config = config
        else:
            user_config = EnvironConfig()

        configuration = Configuration(
            host=user_config.get("API_HOST"),
            username=user_config.get("API_USERNAME"),
            password=user_config.get("API_PASSWORD"),
            api_key={"Authorization": '', "refresh": ''},
            api_key_prefix={"Authorization": "Bearer"}
        )
        configuration.refresh_api_key_hook = refresh_api_key
        api_client = ApiClient(configuration)
        return api_client



# coding: utf-8

"""
    3Di API

    3Di simulation API (latest version: 3.0)   Framework release: 1.0.16   3Di core release: 2.0.11  deployed on:  07:33AM (UTC) on September 04, 2020  # noqa: E501

    The version of the OpenAPI document: 3.0
    Contact: info@nelen-schuurmans.nl
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class TimeseriesRainOverview(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'url': 'str',
        'simulation': 'str',
        'offset': 'int',
        'duration': 'int',
        'interpolate': 'bool',
        'values': 'list[list[float]]',
        'units': 'str',
        'constant': 'bool',
        'uid': 'str',
        'id': 'int'
    }

    attribute_map = {
        'url': 'url',
        'simulation': 'simulation',
        'offset': 'offset',
        'duration': 'duration',
        'interpolate': 'interpolate',
        'values': 'values',
        'units': 'units',
        'constant': 'constant',
        'uid': 'uid',
        'id': 'id'
    }

    def __init__(self, url=None, simulation=None, offset=None, duration=None, interpolate=None, values=None, units=None, constant=None, uid=None, id=None, local_vars_configuration=None):  # noqa: E501
        """TimeseriesRainOverview - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._url = None
        self._simulation = None
        self._offset = None
        self._duration = None
        self._interpolate = None
        self._values = None
        self._units = None
        self._constant = None
        self._uid = None
        self._id = None
        self.discriminator = None

        if url is not None:
            self.url = url
        if simulation is not None:
            self.simulation = simulation
        self.offset = offset
        if duration is not None:
            self.duration = duration
        if interpolate is not None:
            self.interpolate = interpolate
        self.values = values
        self.units = units
        if constant is not None:
            self.constant = constant
        if uid is not None:
            self.uid = uid
        if id is not None:
            self.id = id

    @property
    def url(self):
        """Gets the url of this TimeseriesRainOverview.  # noqa: E501


        :return: The url of this TimeseriesRainOverview.  # noqa: E501
        :rtype: str
        """
        return self._url

    @url.setter
    def url(self, url):
        """Sets the url of this TimeseriesRainOverview.


        :param url: The url of this TimeseriesRainOverview.  # noqa: E501
        :type: str
        """

        self._url = url

    @property
    def simulation(self):
        """Gets the simulation of this TimeseriesRainOverview.  # noqa: E501


        :return: The simulation of this TimeseriesRainOverview.  # noqa: E501
        :rtype: str
        """
        return self._simulation

    @simulation.setter
    def simulation(self, simulation):
        """Sets the simulation of this TimeseriesRainOverview.


        :param simulation: The simulation of this TimeseriesRainOverview.  # noqa: E501
        :type: str
        """

        self._simulation = simulation

    @property
    def offset(self):
        """Gets the offset of this TimeseriesRainOverview.  # noqa: E501

        offset of event in simulation in seconds  # noqa: E501

        :return: The offset of this TimeseriesRainOverview.  # noqa: E501
        :rtype: int
        """
        return self._offset

    @offset.setter
    def offset(self, offset):
        """Sets the offset of this TimeseriesRainOverview.

        offset of event in simulation in seconds  # noqa: E501

        :param offset: The offset of this TimeseriesRainOverview.  # noqa: E501
        :type: int
        """
        if self.local_vars_configuration.client_side_validation and offset is None:  # noqa: E501
            raise ValueError("Invalid value for `offset`, must not be `None`")  # noqa: E501
        if (self.local_vars_configuration.client_side_validation and
                offset is not None and offset > 2147483647):  # noqa: E501
            raise ValueError("Invalid value for `offset`, must be a value less than or equal to `2147483647`")  # noqa: E501
        if (self.local_vars_configuration.client_side_validation and
                offset is not None and offset < 0):  # noqa: E501
            raise ValueError("Invalid value for `offset`, must be a value greater than or equal to `0`")  # noqa: E501

        self._offset = offset

    @property
    def duration(self):
        """Gets the duration of this TimeseriesRainOverview.  # noqa: E501

        event duration in seconds. -9999 is the 'infinite duration' value (only allowed in conjunction with infinite simulations  # noqa: E501

        :return: The duration of this TimeseriesRainOverview.  # noqa: E501
        :rtype: int
        """
        return self._duration

    @duration.setter
    def duration(self, duration):
        """Sets the duration of this TimeseriesRainOverview.

        event duration in seconds. -9999 is the 'infinite duration' value (only allowed in conjunction with infinite simulations  # noqa: E501

        :param duration: The duration of this TimeseriesRainOverview.  # noqa: E501
        :type: int
        """

        self._duration = duration

    @property
    def interpolate(self):
        """Gets the interpolate of this TimeseriesRainOverview.  # noqa: E501


        :return: The interpolate of this TimeseriesRainOverview.  # noqa: E501
        :rtype: bool
        """
        return self._interpolate

    @interpolate.setter
    def interpolate(self, interpolate):
        """Sets the interpolate of this TimeseriesRainOverview.


        :param interpolate: The interpolate of this TimeseriesRainOverview.  # noqa: E501
        :type: bool
        """

        self._interpolate = interpolate

    @property
    def values(self):
        """Gets the values of this TimeseriesRainOverview.  # noqa: E501

        Timeseries provided as a nested list. The inner list consists of exactly 2 values: timestamp, value  # noqa: E501

        :return: The values of this TimeseriesRainOverview.  # noqa: E501
        :rtype: list[list[float]]
        """
        return self._values

    @values.setter
    def values(self, values):
        """Sets the values of this TimeseriesRainOverview.

        Timeseries provided as a nested list. The inner list consists of exactly 2 values: timestamp, value  # noqa: E501

        :param values: The values of this TimeseriesRainOverview.  # noqa: E501
        :type: list[list[float]]
        """
        if self.local_vars_configuration.client_side_validation and values is None:  # noqa: E501
            raise ValueError("Invalid value for `values`, must not be `None`")  # noqa: E501

        self._values = values

    @property
    def units(self):
        """Gets the units of this TimeseriesRainOverview.  # noqa: E501

        m/s is only option for now  # noqa: E501

        :return: The units of this TimeseriesRainOverview.  # noqa: E501
        :rtype: str
        """
        return self._units

    @units.setter
    def units(self, units):
        """Sets the units of this TimeseriesRainOverview.

        m/s is only option for now  # noqa: E501

        :param units: The units of this TimeseriesRainOverview.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and units is None:  # noqa: E501
            raise ValueError("Invalid value for `units`, must not be `None`")  # noqa: E501
        allowed_values = ["m/s"]  # noqa: E501
        if self.local_vars_configuration.client_side_validation and units not in allowed_values:  # noqa: E501
            raise ValueError(
                "Invalid value for `units` ({0}), must be one of {1}"  # noqa: E501
                .format(units, allowed_values)
            )

        self._units = units

    @property
    def constant(self):
        """Gets the constant of this TimeseriesRainOverview.  # noqa: E501


        :return: The constant of this TimeseriesRainOverview.  # noqa: E501
        :rtype: bool
        """
        return self._constant

    @constant.setter
    def constant(self, constant):
        """Sets the constant of this TimeseriesRainOverview.


        :param constant: The constant of this TimeseriesRainOverview.  # noqa: E501
        :type: bool
        """

        self._constant = constant

    @property
    def uid(self):
        """Gets the uid of this TimeseriesRainOverview.  # noqa: E501


        :return: The uid of this TimeseriesRainOverview.  # noqa: E501
        :rtype: str
        """
        return self._uid

    @uid.setter
    def uid(self, uid):
        """Sets the uid of this TimeseriesRainOverview.


        :param uid: The uid of this TimeseriesRainOverview.  # noqa: E501
        :type: str
        """

        self._uid = uid

    @property
    def id(self):
        """Gets the id of this TimeseriesRainOverview.  # noqa: E501


        :return: The id of this TimeseriesRainOverview.  # noqa: E501
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this TimeseriesRainOverview.


        :param id: The id of this TimeseriesRainOverview.  # noqa: E501
        :type: int
        """

        self._id = id

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, TimeseriesRainOverview):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, TimeseriesRainOverview):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

# flake8: noqa

"""
    3Di API

    3Di simulation API (latest version: 3.0)   Framework release: 1.0.10   3Di core release: 2.0.10  deployed on:  09:44AM (UTC) on July 02, 2020  # noqa: E501

    The version of the OpenAPI document: 3.0
    Contact: info@nelen-schuurmans.nl
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

__version__ = "1.0.0"

from openapi_client.aio import rest as rest_iets
from openapi_client import *


# import ApiClient
rest = rest_iets
from openapi_client.aio.api_client import ApiClient




# coding: utf-8

"""
    3Di API

    3Di simulation API (latest version: 3.0)   Framework release: 1.0.16   3Di core release: 2.0.11  deployed on:  07:33AM (UTC) on September 04, 2020  # noqa: E501

    The version of the OpenAPI document: 3.0
    Contact: info@nelen-schuurmans.nl
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class ThreediModel(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'url': 'str',
        'id': 'int',
        'user': 'str',
        'inpy_version': 'str',
        'revision': 'str',
        'revision_hash': 'str',
        'revision_number': 'str',
        'revision_commit_date': 'str',
        'repository_slug': 'str',
        'name': 'str',
        'slug': 'str',
        'disabled': 'bool',
        'epsg': 'int',
        'inp_success': 'bool',
        'description': 'str',
        'storage_space': 'int',
        'storage_space_humanized': 'str',
        'model_ini': 'str',
        'breach_count': 'str',
        'extent_two_d': 'Extent',
        'extent_one_d': 'Extent',
        'extent_zero_d': 'Extent',
        'nodes_count': 'int',
        'lines_count': 'int'
    }

    attribute_map = {
        'url': 'url',
        'id': 'id',
        'user': 'user',
        'inpy_version': 'inpy_version',
        'revision': 'revision',
        'revision_hash': 'revision_hash',
        'revision_number': 'revision_number',
        'revision_commit_date': 'revision_commit_date',
        'repository_slug': 'repository_slug',
        'name': 'name',
        'slug': 'slug',
        'disabled': 'disabled',
        'epsg': 'epsg',
        'inp_success': 'inp_success',
        'description': 'description',
        'storage_space': 'storage_space',
        'storage_space_humanized': 'storage_space_humanized',
        'model_ini': 'model_ini',
        'breach_count': 'breach_count',
        'extent_two_d': 'extent_two_d',
        'extent_one_d': 'extent_one_d',
        'extent_zero_d': 'extent_zero_d',
        'nodes_count': 'nodes_count',
        'lines_count': 'lines_count'
    }

    def __init__(self, url=None, id=None, user=None, inpy_version=None, revision=None, revision_hash=None, revision_number=None, revision_commit_date=None, repository_slug=None, name=None, slug=None, disabled=None, epsg=None, inp_success=None, description=None, storage_space=None, storage_space_humanized=None, model_ini=None, breach_count=None, extent_two_d=None, extent_one_d=None, extent_zero_d=None, nodes_count=None, lines_count=None, local_vars_configuration=None):  # noqa: E501
        """ThreediModel - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._url = None
        self._id = None
        self._user = None
        self._inpy_version = None
        self._revision = None
        self._revision_hash = None
        self._revision_number = None
        self._revision_commit_date = None
        self._repository_slug = None
        self._name = None
        self._slug = None
        self._disabled = None
        self._epsg = None
        self._inp_success = None
        self._description = None
        self._storage_space = None
        self._storage_space_humanized = None
        self._model_ini = None
        self._breach_count = None
        self._extent_two_d = None
        self._extent_one_d = None
        self._extent_zero_d = None
        self._nodes_count = None
        self._lines_count = None
        self.discriminator = None

        if url is not None:
            self.url = url
        if id is not None:
            self.id = id
        if user is not None:
            self.user = user
        self.inpy_version = inpy_version
        self.revision = revision
        if revision_hash is not None:
            self.revision_hash = revision_hash
        if revision_number is not None:
            self.revision_number = revision_number
        if revision_commit_date is not None:
            self.revision_commit_date = revision_commit_date
        if repository_slug is not None:
            self.repository_slug = repository_slug
        if name is not None:
            self.name = name
        self.slug = slug
        if disabled is not None:
            self.disabled = disabled
        self.epsg = epsg
        self.inp_success = inp_success
        self.description = description
        if storage_space is not None:
            self.storage_space = storage_space
        if storage_space_humanized is not None:
            self.storage_space_humanized = storage_space_humanized
        self.model_ini = model_ini
        if breach_count is not None:
            self.breach_count = breach_count
        if extent_two_d is not None:
            self.extent_two_d = extent_two_d
        if extent_one_d is not None:
            self.extent_one_d = extent_one_d
        if extent_zero_d is not None:
            self.extent_zero_d = extent_zero_d
        self.nodes_count = nodes_count
        self.lines_count = lines_count

    @property
    def url(self):
        """Gets the url of this ThreediModel.  # noqa: E501


        :return: The url of this ThreediModel.  # noqa: E501
        :rtype: str
        """
        return self._url

    @url.setter
    def url(self, url):
        """Sets the url of this ThreediModel.


        :param url: The url of this ThreediModel.  # noqa: E501
        :type: str
        """

        self._url = url

    @property
    def id(self):
        """Gets the id of this ThreediModel.  # noqa: E501


        :return: The id of this ThreediModel.  # noqa: E501
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this ThreediModel.


        :param id: The id of this ThreediModel.  # noqa: E501
        :type: int
        """

        self._id = id

    @property
    def user(self):
        """Gets the user of this ThreediModel.  # noqa: E501

        The username of a user  # noqa: E501

        :return: The user of this ThreediModel.  # noqa: E501
        :rtype: str
        """
        return self._user

    @user.setter
    def user(self, user):
        """Sets the user of this ThreediModel.

        The username of a user  # noqa: E501

        :param user: The user of this ThreediModel.  # noqa: E501
        :type: str
        """

        self._user = user

    @property
    def inpy_version(self):
        """Gets the inpy_version of this ThreediModel.  # noqa: E501


        :return: The inpy_version of this ThreediModel.  # noqa: E501
        :rtype: str
        """
        return self._inpy_version

    @inpy_version.setter
    def inpy_version(self, inpy_version):
        """Sets the inpy_version of this ThreediModel.


        :param inpy_version: The inpy_version of this ThreediModel.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and inpy_version is None:  # noqa: E501
            raise ValueError("Invalid value for `inpy_version`, must not be `None`")  # noqa: E501

        self._inpy_version = inpy_version

    @property
    def revision(self):
        """Gets the revision of this ThreediModel.  # noqa: E501


        :return: The revision of this ThreediModel.  # noqa: E501
        :rtype: str
        """
        return self._revision

    @revision.setter
    def revision(self, revision):
        """Sets the revision of this ThreediModel.


        :param revision: The revision of this ThreediModel.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and revision is None:  # noqa: E501
            raise ValueError("Invalid value for `revision`, must not be `None`")  # noqa: E501

        self._revision = revision

    @property
    def revision_hash(self):
        """Gets the revision_hash of this ThreediModel.  # noqa: E501


        :return: The revision_hash of this ThreediModel.  # noqa: E501
        :rtype: str
        """
        return self._revision_hash

    @revision_hash.setter
    def revision_hash(self, revision_hash):
        """Sets the revision_hash of this ThreediModel.


        :param revision_hash: The revision_hash of this ThreediModel.  # noqa: E501
        :type: str
        """

        self._revision_hash = revision_hash

    @property
    def revision_number(self):
        """Gets the revision_number of this ThreediModel.  # noqa: E501


        :return: The revision_number of this ThreediModel.  # noqa: E501
        :rtype: str
        """
        return self._revision_number

    @revision_number.setter
    def revision_number(self, revision_number):
        """Sets the revision_number of this ThreediModel.


        :param revision_number: The revision_number of this ThreediModel.  # noqa: E501
        :type: str
        """

        self._revision_number = revision_number

    @property
    def revision_commit_date(self):
        """Gets the revision_commit_date of this ThreediModel.  # noqa: E501


        :return: The revision_commit_date of this ThreediModel.  # noqa: E501
        :rtype: str
        """
        return self._revision_commit_date

    @revision_commit_date.setter
    def revision_commit_date(self, revision_commit_date):
        """Sets the revision_commit_date of this ThreediModel.


        :param revision_commit_date: The revision_commit_date of this ThreediModel.  # noqa: E501
        :type: str
        """

        self._revision_commit_date = revision_commit_date

    @property
    def repository_slug(self):
        """Gets the repository_slug of this ThreediModel.  # noqa: E501


        :return: The repository_slug of this ThreediModel.  # noqa: E501
        :rtype: str
        """
        return self._repository_slug

    @repository_slug.setter
    def repository_slug(self, repository_slug):
        """Sets the repository_slug of this ThreediModel.


        :param repository_slug: The repository_slug of this ThreediModel.  # noqa: E501
        :type: str
        """

        self._repository_slug = repository_slug

    @property
    def name(self):
        """Gets the name of this ThreediModel.  # noqa: E501


        :return: The name of this ThreediModel.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this ThreediModel.


        :param name: The name of this ThreediModel.  # noqa: E501
        :type: str
        """
        if (self.local_vars_configuration.client_side_validation and
                name is not None and len(name) < 1):
            raise ValueError("Invalid value for `name`, length must be greater than or equal to `1`")  # noqa: E501

        self._name = name

    @property
    def slug(self):
        """Gets the slug of this ThreediModel.  # noqa: E501


        :return: The slug of this ThreediModel.  # noqa: E501
        :rtype: str
        """
        return self._slug

    @slug.setter
    def slug(self, slug):
        """Sets the slug of this ThreediModel.


        :param slug: The slug of this ThreediModel.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and slug is None:  # noqa: E501
            raise ValueError("Invalid value for `slug`, must not be `None`")  # noqa: E501
        if (self.local_vars_configuration.client_side_validation and
                slug is not None and len(slug) > 255):
            raise ValueError("Invalid value for `slug`, length must be less than or equal to `255`")  # noqa: E501
        if (self.local_vars_configuration.client_side_validation and
                slug is not None and len(slug) < 1):
            raise ValueError("Invalid value for `slug`, length must be greater than or equal to `1`")  # noqa: E501
        if (self.local_vars_configuration.client_side_validation and
                slug is not None and not re.search(r'^[-a-zA-Z0-9_]+$', slug)):  # noqa: E501
            raise ValueError(r"Invalid value for `slug`, must be a follow pattern or equal to `/^[-a-zA-Z0-9_]+$/`")  # noqa: E501

        self._slug = slug

    @property
    def disabled(self):
        """Gets the disabled of this ThreediModel.  # noqa: E501

        Disable the model.  # noqa: E501

        :return: The disabled of this ThreediModel.  # noqa: E501
        :rtype: bool
        """
        return self._disabled

    @disabled.setter
    def disabled(self, disabled):
        """Sets the disabled of this ThreediModel.

        Disable the model.  # noqa: E501

        :param disabled: The disabled of this ThreediModel.  # noqa: E501
        :type: bool
        """

        self._disabled = disabled

    @property
    def epsg(self):
        """Gets the epsg of this ThreediModel.  # noqa: E501


        :return: The epsg of this ThreediModel.  # noqa: E501
        :rtype: int
        """
        return self._epsg

    @epsg.setter
    def epsg(self, epsg):
        """Sets the epsg of this ThreediModel.


        :param epsg: The epsg of this ThreediModel.  # noqa: E501
        :type: int
        """
        if self.local_vars_configuration.client_side_validation and epsg is None:  # noqa: E501
            raise ValueError("Invalid value for `epsg`, must not be `None`")  # noqa: E501
        if (self.local_vars_configuration.client_side_validation and
                epsg is not None and epsg > 2147483647):  # noqa: E501
            raise ValueError("Invalid value for `epsg`, must be a value less than or equal to `2147483647`")  # noqa: E501
        if (self.local_vars_configuration.client_side_validation and
                epsg is not None and epsg < -2147483648):  # noqa: E501
            raise ValueError("Invalid value for `epsg`, must be a value greater than or equal to `-2147483648`")  # noqa: E501

        self._epsg = epsg

    @property
    def inp_success(self):
        """Gets the inp_success of this ThreediModel.  # noqa: E501

        signals success of generate inp/sewerage_inp/...  # noqa: E501

        :return: The inp_success of this ThreediModel.  # noqa: E501
        :rtype: bool
        """
        return self._inp_success

    @inp_success.setter
    def inp_success(self, inp_success):
        """Sets the inp_success of this ThreediModel.

        signals success of generate inp/sewerage_inp/...  # noqa: E501

        :param inp_success: The inp_success of this ThreediModel.  # noqa: E501
        :type: bool
        """

        self._inp_success = inp_success

    @property
    def description(self):
        """Gets the description of this ThreediModel.  # noqa: E501

        Please describe the model here...  # noqa: E501

        :return: The description of this ThreediModel.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this ThreediModel.

        Please describe the model here...  # noqa: E501

        :param description: The description of this ThreediModel.  # noqa: E501
        :type: str
        """

        self._description = description

    @property
    def storage_space(self):
        """Gets the storage_space of this ThreediModel.  # noqa: E501

        Automatically filled after inp generation.  # noqa: E501

        :return: The storage_space of this ThreediModel.  # noqa: E501
        :rtype: int
        """
        return self._storage_space

    @storage_space.setter
    def storage_space(self, storage_space):
        """Sets the storage_space of this ThreediModel.

        Automatically filled after inp generation.  # noqa: E501

        :param storage_space: The storage_space of this ThreediModel.  # noqa: E501
        :type: int
        """
        if (self.local_vars_configuration.client_side_validation and
                storage_space is not None and storage_space > 9223372036854775807):  # noqa: E501
            raise ValueError("Invalid value for `storage_space`, must be a value less than or equal to `9223372036854775807`")  # noqa: E501
        if (self.local_vars_configuration.client_side_validation and
                storage_space is not None and storage_space < -9223372036854775808):  # noqa: E501
            raise ValueError("Invalid value for `storage_space`, must be a value greater than or equal to `-9223372036854775808`")  # noqa: E501

        self._storage_space = storage_space

    @property
    def storage_space_humanized(self):
        """Gets the storage_space_humanized of this ThreediModel.  # noqa: E501


        :return: The storage_space_humanized of this ThreediModel.  # noqa: E501
        :rtype: str
        """
        return self._storage_space_humanized

    @storage_space_humanized.setter
    def storage_space_humanized(self, storage_space_humanized):
        """Sets the storage_space_humanized of this ThreediModel.


        :param storage_space_humanized: The storage_space_humanized of this ThreediModel.  # noqa: E501
        :type: str
        """

        self._storage_space_humanized = storage_space_humanized

    @property
    def model_ini(self):
        """Gets the model_ini of this ThreediModel.  # noqa: E501


        :return: The model_ini of this ThreediModel.  # noqa: E501
        :rtype: str
        """
        return self._model_ini

    @model_ini.setter
    def model_ini(self, model_ini):
        """Sets the model_ini of this ThreediModel.


        :param model_ini: The model_ini of this ThreediModel.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and model_ini is None:  # noqa: E501
            raise ValueError("Invalid value for `model_ini`, must not be `None`")  # noqa: E501
        if (self.local_vars_configuration.client_side_validation and
                model_ini is not None and len(model_ini) > 512):
            raise ValueError("Invalid value for `model_ini`, length must be less than or equal to `512`")  # noqa: E501
        if (self.local_vars_configuration.client_side_validation and
                model_ini is not None and len(model_ini) < 1):
            raise ValueError("Invalid value for `model_ini`, length must be greater than or equal to `1`")  # noqa: E501

        self._model_ini = model_ini

    @property
    def breach_count(self):
        """Gets the breach_count of this ThreediModel.  # noqa: E501


        :return: The breach_count of this ThreediModel.  # noqa: E501
        :rtype: str
        """
        return self._breach_count

    @breach_count.setter
    def breach_count(self, breach_count):
        """Sets the breach_count of this ThreediModel.


        :param breach_count: The breach_count of this ThreediModel.  # noqa: E501
        :type: str
        """

        self._breach_count = breach_count

    @property
    def extent_two_d(self):
        """Gets the extent_two_d of this ThreediModel.  # noqa: E501


        :return: The extent_two_d of this ThreediModel.  # noqa: E501
        :rtype: Extent
        """
        return self._extent_two_d

    @extent_two_d.setter
    def extent_two_d(self, extent_two_d):
        """Sets the extent_two_d of this ThreediModel.


        :param extent_two_d: The extent_two_d of this ThreediModel.  # noqa: E501
        :type: Extent
        """

        self._extent_two_d = extent_two_d

    @property
    def extent_one_d(self):
        """Gets the extent_one_d of this ThreediModel.  # noqa: E501


        :return: The extent_one_d of this ThreediModel.  # noqa: E501
        :rtype: Extent
        """
        return self._extent_one_d

    @extent_one_d.setter
    def extent_one_d(self, extent_one_d):
        """Sets the extent_one_d of this ThreediModel.


        :param extent_one_d: The extent_one_d of this ThreediModel.  # noqa: E501
        :type: Extent
        """

        self._extent_one_d = extent_one_d

    @property
    def extent_zero_d(self):
        """Gets the extent_zero_d of this ThreediModel.  # noqa: E501


        :return: The extent_zero_d of this ThreediModel.  # noqa: E501
        :rtype: Extent
        """
        return self._extent_zero_d

    @extent_zero_d.setter
    def extent_zero_d(self, extent_zero_d):
        """Sets the extent_zero_d of this ThreediModel.


        :param extent_zero_d: The extent_zero_d of this ThreediModel.  # noqa: E501
        :type: Extent
        """

        self._extent_zero_d = extent_zero_d

    @property
    def nodes_count(self):
        """Gets the nodes_count of this ThreediModel.  # noqa: E501


        :return: The nodes_count of this ThreediModel.  # noqa: E501
        :rtype: int
        """
        return self._nodes_count

    @nodes_count.setter
    def nodes_count(self, nodes_count):
        """Sets the nodes_count of this ThreediModel.


        :param nodes_count: The nodes_count of this ThreediModel.  # noqa: E501
        :type: int
        """
        if (self.local_vars_configuration.client_side_validation and
                nodes_count is not None and nodes_count > 2147483647):  # noqa: E501
            raise ValueError("Invalid value for `nodes_count`, must be a value less than or equal to `2147483647`")  # noqa: E501
        if (self.local_vars_configuration.client_side_validation and
                nodes_count is not None and nodes_count < -2147483648):  # noqa: E501
            raise ValueError("Invalid value for `nodes_count`, must be a value greater than or equal to `-2147483648`")  # noqa: E501

        self._nodes_count = nodes_count

    @property
    def lines_count(self):
        """Gets the lines_count of this ThreediModel.  # noqa: E501


        :return: The lines_count of this ThreediModel.  # noqa: E501
        :rtype: int
        """
        return self._lines_count

    @lines_count.setter
    def lines_count(self, lines_count):
        """Sets the lines_count of this ThreediModel.


        :param lines_count: The lines_count of this ThreediModel.  # noqa: E501
        :type: int
        """
        if (self.local_vars_configuration.client_side_validation and
                lines_count is not None and lines_count > 2147483647):  # noqa: E501
            raise ValueError("Invalid value for `lines_count`, must be a value less than or equal to `2147483647`")  # noqa: E501
        if (self.local_vars_configuration.client_side_validation and
                lines_count is not None and lines_count < -2147483648):  # noqa: E501
            raise ValueError("Invalid value for `lines_count`, must be a value greater than or equal to `-2147483648`")  # noqa: E501

        self._lines_count = lines_count

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ThreediModel):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ThreediModel):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    3Di API

    3Di simulation API (latest version: 3.0)   Framework release: 1.0.16   3Di core release: 2.0.11  deployed on:  07:33AM (UTC) on September 04, 2020  # noqa: E501

    The version of the OpenAPI document: 3.0
    Contact: info@nelen-schuurmans.nl
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class MeasureSpecification(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'id': 'int',
        'name': 'str',
        'locations': 'list[MeasureLocation]',
        'variable': 'str',
        'operator': 'str'
    }

    attribute_map = {
        'id': 'id',
        'name': 'name',
        'locations': 'locations',
        'variable': 'variable',
        'operator': 'operator'
    }

    def __init__(self, id=None, name=None, locations=None, variable=None, operator=None, local_vars_configuration=None):  # noqa: E501
        """MeasureSpecification - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._id = None
        self._name = None
        self._locations = None
        self._variable = None
        self._operator = None
        self.discriminator = None

        if id is not None:
            self.id = id
        if name is not None:
            self.name = name
        self.locations = locations
        self.variable = variable
        self.operator = operator

    @property
    def id(self):
        """Gets the id of this MeasureSpecification.  # noqa: E501


        :return: The id of this MeasureSpecification.  # noqa: E501
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this MeasureSpecification.


        :param id: The id of this MeasureSpecification.  # noqa: E501
        :type: int
        """

        self._id = id

    @property
    def name(self):
        """Gets the name of this MeasureSpecification.  # noqa: E501


        :return: The name of this MeasureSpecification.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this MeasureSpecification.


        :param name: The name of this MeasureSpecification.  # noqa: E501
        :type: str
        """
        if (self.local_vars_configuration.client_side_validation and
                name is not None and len(name) > 50):
            raise ValueError("Invalid value for `name`, length must be less than or equal to `50`")  # noqa: E501

        self._name = name

    @property
    def locations(self):
        """Gets the locations of this MeasureSpecification.  # noqa: E501


        :return: The locations of this MeasureSpecification.  # noqa: E501
        :rtype: list[MeasureLocation]
        """
        return self._locations

    @locations.setter
    def locations(self, locations):
        """Sets the locations of this MeasureSpecification.


        :param locations: The locations of this MeasureSpecification.  # noqa: E501
        :type: list[MeasureLocation]
        """
        if self.local_vars_configuration.client_side_validation and locations is None:  # noqa: E501
            raise ValueError("Invalid value for `locations`, must not be `None`")  # noqa: E501

        self._locations = locations

    @property
    def variable(self):
        """Gets the variable of this MeasureSpecification.  # noqa: E501

        measurement variable, one of the following options:  s1 (waterlevel) vol1 (volume) q (discharge) u1 (velocity)   # noqa: E501

        :return: The variable of this MeasureSpecification.  # noqa: E501
        :rtype: str
        """
        return self._variable

    @variable.setter
    def variable(self, variable):
        """Sets the variable of this MeasureSpecification.

        measurement variable, one of the following options:  s1 (waterlevel) vol1 (volume) q (discharge) u1 (velocity)   # noqa: E501

        :param variable: The variable of this MeasureSpecification.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and variable is None:  # noqa: E501
            raise ValueError("Invalid value for `variable`, must not be `None`")  # noqa: E501
        allowed_values = ["s1", "vol1", "q", "u1"]  # noqa: E501
        if self.local_vars_configuration.client_side_validation and variable not in allowed_values:  # noqa: E501
            raise ValueError(
                "Invalid value for `variable` ({0}), must be one of {1}"  # noqa: E501
                .format(variable, allowed_values)
            )

        self._variable = variable

    @property
    def operator(self):
        """Gets the operator of this MeasureSpecification.  # noqa: E501

        e.g. >, <, >=, <=  # noqa: E501

        :return: The operator of this MeasureSpecification.  # noqa: E501
        :rtype: str
        """
        return self._operator

    @operator.setter
    def operator(self, operator):
        """Sets the operator of this MeasureSpecification.

        e.g. >, <, >=, <=  # noqa: E501

        :param operator: The operator of this MeasureSpecification.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and operator is None:  # noqa: E501
            raise ValueError("Invalid value for `operator`, must not be `None`")  # noqa: E501
        allowed_values = [">", ">=", "<", "<="]  # noqa: E501
        if self.local_vars_configuration.client_side_validation and operator not in allowed_values:  # noqa: E501
            raise ValueError(
                "Invalid value for `operator` ({0}), must be one of {1}"  # noqa: E501
                .format(operator, allowed_values)
            )

        self._operator = operator

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, MeasureSpecification):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, MeasureSpecification):
            return True

        return self.to_dict() != other.to_dict()



import os


class ConfigError(Exception):
    pass


class Config:
    def __init__(self, env_file=None):
        self.file_values = {}
        if env_file is not None and os.path.isfile(env_file):
            self.file_values = self._read_file(env_file)
        else:
            raise ConfigError("Could not find %s", env_file)

    def __call__(self, key):
        return self.get(key)

    def get(self, key):
        if key in self.file_values:
            return self.file_values[key]
        raise KeyError("Config '%s' is missing." % key)

    def _read_file(self, file_name):
        file_values = {}
        with open(file_name) as input_file:
            for line in input_file.readlines():
                line = line.strip()
                if "=" in line and not line.startswith("#"):
                    key, value = line.split("=", 1)
                    key = key.strip()
                    value = value.strip().strip("\"'")
                    file_values[key] = value
        return file_values


class EnvironConfig:
    def __call__(self, key):
        return self.get(key)

    def get(self, key):
        if key in os.environ:
            value = os.environ[key]
            return value
        raise KeyError("Config '%s' is missing in environment variables" % key)



# coding: utf-8

"""
    3Di API

    3Di simulation API (latest version: 3.0)   Framework release: 1.0.16   3Di core release: 2.0.11  deployed on:  07:33AM (UTC) on September 04, 2020  # noqa: E501

    The version of the OpenAPI document: 3.0
    Contact: info@nelen-schuurmans.nl
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class UsageStatistics(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'total_sessions': 'int',
        'total_duration': 'int',
        'avg_duration': 'float',
        'min_duration': 'int',
        'max_duration': 'int',
        'duration_humanized': 'str'
    }

    attribute_map = {
        'total_sessions': 'total_sessions',
        'total_duration': 'total_duration',
        'avg_duration': 'avg_duration',
        'min_duration': 'min_duration',
        'max_duration': 'max_duration',
        'duration_humanized': 'duration_humanized'
    }

    def __init__(self, total_sessions=None, total_duration=None, avg_duration=None, min_duration=None, max_duration=None, duration_humanized=None, local_vars_configuration=None):  # noqa: E501
        """UsageStatistics - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._total_sessions = None
        self._total_duration = None
        self._avg_duration = None
        self._min_duration = None
        self._max_duration = None
        self._duration_humanized = None
        self.discriminator = None

        self.total_sessions = total_sessions
        self.total_duration = total_duration
        self.avg_duration = avg_duration
        self.min_duration = min_duration
        self.max_duration = max_duration
        self.duration_humanized = duration_humanized

    @property
    def total_sessions(self):
        """Gets the total_sessions of this UsageStatistics.  # noqa: E501


        :return: The total_sessions of this UsageStatistics.  # noqa: E501
        :rtype: int
        """
        return self._total_sessions

    @total_sessions.setter
    def total_sessions(self, total_sessions):
        """Sets the total_sessions of this UsageStatistics.


        :param total_sessions: The total_sessions of this UsageStatistics.  # noqa: E501
        :type: int
        """
        if self.local_vars_configuration.client_side_validation and total_sessions is None:  # noqa: E501
            raise ValueError("Invalid value for `total_sessions`, must not be `None`")  # noqa: E501

        self._total_sessions = total_sessions

    @property
    def total_duration(self):
        """Gets the total_duration of this UsageStatistics.  # noqa: E501


        :return: The total_duration of this UsageStatistics.  # noqa: E501
        :rtype: int
        """
        return self._total_duration

    @total_duration.setter
    def total_duration(self, total_duration):
        """Sets the total_duration of this UsageStatistics.


        :param total_duration: The total_duration of this UsageStatistics.  # noqa: E501
        :type: int
        """
        if self.local_vars_configuration.client_side_validation and total_duration is None:  # noqa: E501
            raise ValueError("Invalid value for `total_duration`, must not be `None`")  # noqa: E501

        self._total_duration = total_duration

    @property
    def avg_duration(self):
        """Gets the avg_duration of this UsageStatistics.  # noqa: E501


        :return: The avg_duration of this UsageStatistics.  # noqa: E501
        :rtype: float
        """
        return self._avg_duration

    @avg_duration.setter
    def avg_duration(self, avg_duration):
        """Sets the avg_duration of this UsageStatistics.


        :param avg_duration: The avg_duration of this UsageStatistics.  # noqa: E501
        :type: float
        """
        if self.local_vars_configuration.client_side_validation and avg_duration is None:  # noqa: E501
            raise ValueError("Invalid value for `avg_duration`, must not be `None`")  # noqa: E501

        self._avg_duration = avg_duration

    @property
    def min_duration(self):
        """Gets the min_duration of this UsageStatistics.  # noqa: E501


        :return: The min_duration of this UsageStatistics.  # noqa: E501
        :rtype: int
        """
        return self._min_duration

    @min_duration.setter
    def min_duration(self, min_duration):
        """Sets the min_duration of this UsageStatistics.


        :param min_duration: The min_duration of this UsageStatistics.  # noqa: E501
        :type: int
        """
        if self.local_vars_configuration.client_side_validation and min_duration is None:  # noqa: E501
            raise ValueError("Invalid value for `min_duration`, must not be `None`")  # noqa: E501

        self._min_duration = min_duration

    @property
    def max_duration(self):
        """Gets the max_duration of this UsageStatistics.  # noqa: E501


        :return: The max_duration of this UsageStatistics.  # noqa: E501
        :rtype: int
        """
        return self._max_duration

    @max_duration.setter
    def max_duration(self, max_duration):
        """Sets the max_duration of this UsageStatistics.


        :param max_duration: The max_duration of this UsageStatistics.  # noqa: E501
        :type: int
        """
        if self.local_vars_configuration.client_side_validation and max_duration is None:  # noqa: E501
            raise ValueError("Invalid value for `max_duration`, must not be `None`")  # noqa: E501

        self._max_duration = max_duration

    @property
    def duration_humanized(self):
        """Gets the duration_humanized of this UsageStatistics.  # noqa: E501


        :return: The duration_humanized of this UsageStatistics.  # noqa: E501
        :rtype: str
        """
        return self._duration_humanized

    @duration_humanized.setter
    def duration_humanized(self, duration_humanized):
        """Sets the duration_humanized of this UsageStatistics.


        :param duration_humanized: The duration_humanized of this UsageStatistics.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and duration_humanized is None:  # noqa: E501
            raise ValueError("Invalid value for `duration_humanized`, must not be `None`")  # noqa: E501
        if (self.local_vars_configuration.client_side_validation and
                duration_humanized is not None and len(duration_humanized) < 1):
            raise ValueError("Invalid value for `duration_humanized`, length must be greater than or equal to `1`")  # noqa: E501

        self._duration_humanized = duration_humanized

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, UsageStatistics):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, UsageStatistics):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    3Di API

    3Di simulation API (latest version: 3.0)   Framework release: 1.0.16   3Di core release: 2.0.11  deployed on:  07:33AM (UTC) on September 04, 2020  # noqa: E501

    The version of the OpenAPI document: 3.0
    Contact: info@nelen-schuurmans.nl
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

import re  # noqa: F401

# python 2 and python 3 compatibility library
import six

from openapi_client.api_client import ApiClient
from openapi_client.exceptions import (  # noqa: F401
    ApiTypeError,
    ApiValueError
)


class AuthApi(object):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

    def auth_profile_list(self, **kwargs):  # noqa: E501
        """auth_profile_list  # noqa: E501

        Read-only API endpoint for viewing user details for current user  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.auth_profile_list(async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param int limit: Number of results to return per page.
        :param int offset: The initial index from which to return the results.
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: User
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        return self.auth_profile_list_with_http_info(**kwargs)  # noqa: E501

    def auth_profile_list_with_http_info(self, **kwargs):  # noqa: E501
        """auth_profile_list  # noqa: E501

        Read-only API endpoint for viewing user details for current user  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.auth_profile_list_with_http_info(async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param int limit: Number of results to return per page.
        :param int offset: The initial index from which to return the results.
        :param _return_http_data_only: response data without head status code
                                       and headers
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: tuple(User, status_code(int), headers(HTTPHeaderDict))
                 If the method is called asynchronously,
                 returns the request thread.
        """

        local_var_params = locals()

        all_params = [
            'limit',
            'offset'
        ]
        all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout'
            ]
        )

        for key, val in six.iteritems(local_var_params['kwargs']):
            if key not in all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method auth_profile_list" % key
                )
            local_var_params[key] = val
        del local_var_params['kwargs']

        collection_formats = {}

        path_params = {}

        query_params = []
        if 'limit' in local_var_params and local_var_params['limit'] is not None:  # noqa: E501
            query_params.append(('limit', local_var_params['limit']))  # noqa: E501
        if 'offset' in local_var_params and local_var_params['offset'] is not None:  # noqa: E501
            query_params.append(('offset', local_var_params['offset']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['Bearer']  # noqa: E501

        return self.api_client.call_api(
            '/auth/profile/', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='User',  # noqa: E501
            auth_settings=auth_settings,
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats=collection_formats)

    def auth_refresh_token_create(self, data, **kwargs):  # noqa: E501
        """Refresh API authentication token by refresh_token.  # noqa: E501

        Post \"refresh\" token as \"refresh\".  The return \"access\" token needs to be sent as HTTP authentication header with every request with 'Bearer ' prefix (case-sensitive).  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.auth_refresh_token_create(data, async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param Refresh data: (required)
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Tokens
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        return self.auth_refresh_token_create_with_http_info(data, **kwargs)  # noqa: E501

    def auth_refresh_token_create_with_http_info(self, data, **kwargs):  # noqa: E501
        """Refresh API authentication token by refresh_token.  # noqa: E501

        Post \"refresh\" token as \"refresh\".  The return \"access\" token needs to be sent as HTTP authentication header with every request with 'Bearer ' prefix (case-sensitive).  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.auth_refresh_token_create_with_http_info(data, async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param Refresh data: (required)
        :param _return_http_data_only: response data without head status code
                                       and headers
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: tuple(Tokens, status_code(int), headers(HTTPHeaderDict))
                 If the method is called asynchronously,
                 returns the request thread.
        """

        local_var_params = locals()

        all_params = [
            'data'
        ]
        all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout'
            ]
        )

        for key, val in six.iteritems(local_var_params['kwargs']):
            if key not in all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method auth_refresh_token_create" % key
                )
            local_var_params[key] = val
        del local_var_params['kwargs']
        # verify the required parameter 'data' is set
        if self.api_client.client_side_validation and ('data' not in local_var_params or  # noqa: E501
                                                        local_var_params['data'] is None):  # noqa: E501
            raise ApiValueError("Missing the required parameter `data` when calling `auth_refresh_token_create`")  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'data' in local_var_params:
            body_params = local_var_params['data']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['Bearer']  # noqa: E501

        return self.api_client.call_api(
            '/auth/refresh-token/', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='Tokens',  # noqa: E501
            auth_settings=auth_settings,
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats=collection_formats)

    def auth_token_create(self, data, **kwargs):  # noqa: E501
        """Get an API authentication JWT token.  # noqa: E501

        Post your username & password to get an \"access\" and \"refresh\" JWT token.  The \"access\" token needs to be sent as HTTP authentication header with every request with a 'Bearer ' prefix (case-sensitive).  Note that the \"access\" token expires after some time.  The \"refresh\" token can be used to get a new access token via the \"refresh\" endpoint.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.auth_token_create(data, async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param Authenticate data: (required)
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Tokens
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        return self.auth_token_create_with_http_info(data, **kwargs)  # noqa: E501

    def auth_token_create_with_http_info(self, data, **kwargs):  # noqa: E501
        """Get an API authentication JWT token.  # noqa: E501

        Post your username & password to get an \"access\" and \"refresh\" JWT token.  The \"access\" token needs to be sent as HTTP authentication header with every request with a 'Bearer ' prefix (case-sensitive).  Note that the \"access\" token expires after some time.  The \"refresh\" token can be used to get a new access token via the \"refresh\" endpoint.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.auth_token_create_with_http_info(data, async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param Authenticate data: (required)
        :param _return_http_data_only: response data without head status code
                                       and headers
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: tuple(Tokens, status_code(int), headers(HTTPHeaderDict))
                 If the method is called asynchronously,
                 returns the request thread.
        """

        local_var_params = locals()

        all_params = [
            'data'
        ]
        all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout'
            ]
        )

        for key, val in six.iteritems(local_var_params['kwargs']):
            if key not in all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method auth_token_create" % key
                )
            local_var_params[key] = val
        del local_var_params['kwargs']
        # verify the required parameter 'data' is set
        if self.api_client.client_side_validation and ('data' not in local_var_params or  # noqa: E501
                                                        local_var_params['data'] is None):  # noqa: E501
            raise ApiValueError("Missing the required parameter `data` when calling `auth_token_create`")  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'data' in local_var_params:
            body_params = local_var_params['data']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['Bearer']  # noqa: E501

        return self.api_client.call_api(
            '/auth/token/', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='Tokens',  # noqa: E501
            auth_settings=auth_settings,
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats=collection_formats)

    def auth_users_list(self, **kwargs):  # noqa: E501
        """Read-only API endpoint for viewing user details.  # noqa: E501

        - superusers and staff users might view all users - user with a manage role might view all users for the   organisations they have the manage role for - regular users might view their user details  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.auth_users_list(async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param str username:
        :param str username__contains:
        :param str username__icontains:
        :param str username__in: Multiple values may be separated by commas.
        :param str username__startswith:
        :param str username__istartswith:
        :param str username__endswith:
        :param str username__regex:
        :param str email:
        :param str email__contains:
        :param str email__icontains:
        :param str email__in: Multiple values may be separated by commas.
        :param str email__startswith:
        :param str email__istartswith:
        :param str email__endswith:
        :param str email__regex:
        :param int limit: Number of results to return per page.
        :param int offset: The initial index from which to return the results.
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: InlineResponse200
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        return self.auth_users_list_with_http_info(**kwargs)  # noqa: E501

    def auth_users_list_with_http_info(self, **kwargs):  # noqa: E501
        """Read-only API endpoint for viewing user details.  # noqa: E501

        - superusers and staff users might view all users - user with a manage role might view all users for the   organisations they have the manage role for - regular users might view their user details  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.auth_users_list_with_http_info(async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param str username:
        :param str username__contains:
        :param str username__icontains:
        :param str username__in: Multiple values may be separated by commas.
        :param str username__startswith:
        :param str username__istartswith:
        :param str username__endswith:
        :param str username__regex:
        :param str email:
        :param str email__contains:
        :param str email__icontains:
        :param str email__in: Multiple values may be separated by commas.
        :param str email__startswith:
        :param str email__istartswith:
        :param str email__endswith:
        :param str email__regex:
        :param int limit: Number of results to return per page.
        :param int offset: The initial index from which to return the results.
        :param _return_http_data_only: response data without head status code
                                       and headers
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: tuple(InlineResponse200, status_code(int), headers(HTTPHeaderDict))
                 If the method is called asynchronously,
                 returns the request thread.
        """

        local_var_params = locals()

        all_params = [
            'username',
            'username__contains',
            'username__icontains',
            'username__in',
            'username__startswith',
            'username__istartswith',
            'username__endswith',
            'username__regex',
            'email',
            'email__contains',
            'email__icontains',
            'email__in',
            'email__startswith',
            'email__istartswith',
            'email__endswith',
            'email__regex',
            'limit',
            'offset'
        ]
        all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout'
            ]
        )

        for key, val in six.iteritems(local_var_params['kwargs']):
            if key not in all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method auth_users_list" % key
                )
            local_var_params[key] = val
        del local_var_params['kwargs']

        collection_formats = {}

        path_params = {}

        query_params = []
        if 'username' in local_var_params and local_var_params['username'] is not None:  # noqa: E501
            query_params.append(('username', local_var_params['username']))  # noqa: E501
        if 'username__contains' in local_var_params and local_var_params['username__contains'] is not None:  # noqa: E501
            query_params.append(('username__contains', local_var_params['username__contains']))  # noqa: E501
        if 'username__icontains' in local_var_params and local_var_params['username__icontains'] is not None:  # noqa: E501
            query_params.append(('username__icontains', local_var_params['username__icontains']))  # noqa: E501
        if 'username__in' in local_var_params and local_var_params['username__in'] is not None:  # noqa: E501
            query_params.append(('username__in', local_var_params['username__in']))  # noqa: E501
        if 'username__startswith' in local_var_params and local_var_params['username__startswith'] is not None:  # noqa: E501
            query_params.append(('username__startswith', local_var_params['username__startswith']))  # noqa: E501
        if 'username__istartswith' in local_var_params and local_var_params['username__istartswith'] is not None:  # noqa: E501
            query_params.append(('username__istartswith', local_var_params['username__istartswith']))  # noqa: E501
        if 'username__endswith' in local_var_params and local_var_params['username__endswith'] is not None:  # noqa: E501
            query_params.append(('username__endswith', local_var_params['username__endswith']))  # noqa: E501
        if 'username__regex' in local_var_params and local_var_params['username__regex'] is not None:  # noqa: E501
            query_params.append(('username__regex', local_var_params['username__regex']))  # noqa: E501
        if 'email' in local_var_params and local_var_params['email'] is not None:  # noqa: E501
            query_params.append(('email', local_var_params['email']))  # noqa: E501
        if 'email__contains' in local_var_params and local_var_params['email__contains'] is not None:  # noqa: E501
            query_params.append(('email__contains', local_var_params['email__contains']))  # noqa: E501
        if 'email__icontains' in local_var_params and local_var_params['email__icontains'] is not None:  # noqa: E501
            query_params.append(('email__icontains', local_var_params['email__icontains']))  # noqa: E501
        if 'email__in' in local_var_params and local_var_params['email__in'] is not None:  # noqa: E501
            query_params.append(('email__in', local_var_params['email__in']))  # noqa: E501
        if 'email__startswith' in local_var_params and local_var_params['email__startswith'] is not None:  # noqa: E501
            query_params.append(('email__startswith', local_var_params['email__startswith']))  # noqa: E501
        if 'email__istartswith' in local_var_params and local_var_params['email__istartswith'] is not None:  # noqa: E501
            query_params.append(('email__istartswith', local_var_params['email__istartswith']))  # noqa: E501
        if 'email__endswith' in local_var_params and local_var_params['email__endswith'] is not None:  # noqa: E501
            query_params.append(('email__endswith', local_var_params['email__endswith']))  # noqa: E501
        if 'email__regex' in local_var_params and local_var_params['email__regex'] is not None:  # noqa: E501
            query_params.append(('email__regex', local_var_params['email__regex']))  # noqa: E501
        if 'limit' in local_var_params and local_var_params['limit'] is not None:  # noqa: E501
            query_params.append(('limit', local_var_params['limit']))  # noqa: E501
        if 'offset' in local_var_params and local_var_params['offset'] is not None:  # noqa: E501
            query_params.append(('offset', local_var_params['offset']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['Bearer']  # noqa: E501

        return self.api_client.call_api(
            '/auth/users/', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='InlineResponse200',  # noqa: E501
            auth_settings=auth_settings,
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats=collection_formats)

    def auth_users_read(self, id, **kwargs):  # noqa: E501
        """auth_users_read  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.auth_users_read(id, async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param int id: A unique integer value identifying this user. (required)
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: User
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        return self.auth_users_read_with_http_info(id, **kwargs)  # noqa: E501

    def auth_users_read_with_http_info(self, id, **kwargs):  # noqa: E501
        """auth_users_read  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.auth_users_read_with_http_info(id, async_req=True)
        >>> result = thread.get()

        :param async_req bool: execute request asynchronously
        :param int id: A unique integer value identifying this user. (required)
        :param _return_http_data_only: response data without head status code
                                       and headers
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: tuple(User, status_code(int), headers(HTTPHeaderDict))
                 If the method is called asynchronously,
                 returns the request thread.
        """

        local_var_params = locals()

        all_params = [
            'id'
        ]
        all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout'
            ]
        )

        for key, val in six.iteritems(local_var_params['kwargs']):
            if key not in all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method auth_users_read" % key
                )
            local_var_params[key] = val
        del local_var_params['kwargs']
        # verify the required parameter 'id' is set
        if self.api_client.client_side_validation and ('id' not in local_var_params or  # noqa: E501
                                                        local_var_params['id'] is None):  # noqa: E501
            raise ApiValueError("Missing the required parameter `id` when calling `auth_users_read`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'id' in local_var_params:
            path_params['id'] = local_var_params['id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['Bearer']  # noqa: E501

        return self.api_client.call_api(
            '/auth/users/{id}/', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='User',  # noqa: E501
            auth_settings=auth_settings,
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats=collection_formats)



import asyncio
import base64
import hashlib
import logging
import re
from concurrent.futures import ThreadPoolExecutor
from functools import partial
from pathlib import Path
from typing import Optional, Tuple
from urllib.parse import urlparse

import aiofiles
import aiofiles.os
import aiohttp

from openapi_client.exceptions import ApiException

CONTENT_RANGE_REGEXP = re.compile(r"^bytes (\d+)-(\d+)/(\d+|\*)$")
RETRY_STATUSES = frozenset({413, 429, 503})  # like in urllib3
DEFAULT_CONN_LIMIT = 4  # for downloads only (which are parrallel)


logger = logging.getLogger(__name__)


async def _request_with_retry(request_coroutine, retries, backoff_factor):
    """Call a request coroutine and retry on ClientError or on 413/429/503.

    The default retry policy has 3 retries with 1, 2, 4 second intervals.
    """
    assert retries > 0
    for attempt in range(retries):
        if attempt > 0:
            backoff = backoff_factor * 2 ** (attempt - 1)
            logger.warning(
                "Retry attempt {}, waiting {} seconds...".format(attempt, backoff)
            )
            await asyncio.sleep(backoff)

        try:
            response = await request_coroutine()
            await response.read()
        except (aiohttp.ClientError, asyncio.exceptions.TimeoutError):
            if attempt == retries - 1:
                raise  # propagate ClientError in case no retries left
        else:
            if response.status not in RETRY_STATUSES:
                return response  # on all non-retry statuses: return response

    return response  # retries exceeded; return the (possibly error) response


async def download_file(
    url: str,
    target: Path,
    chunk_size: int = 16777216,
    timeout: float = 300.0,
    connector: Optional[aiohttp.BaseConnector] = None,
    executor: Optional[ThreadPoolExecutor] = None,
    retries: int = 3,
    backoff_factor: float = 1.0,
) -> Tuple[Path, int]:
    """Download a file to a specified path on disk.

    It is assumed that the file server supports multipart downloads (range
    requests).

    Args:
        url: The url to retrieve.
        target: The location to copy to. If this is an existing file, it is
            overwritten. If it is a directory, a filename is generated from
            the filename in the url.
        chunk_size: The number of bytes per request. Default: 16MB.
        timeout: The timeout of the download of a single chunk in seconds.
        connector: An optional aiohttp connector to support connection pooling.
            If not supplied, a default TCPConnector is instantiated with a pool
            size (limit) of 4.
        executor: The ThreadPoolExecutor to execute local
            file I/O in. If not supplied, default executor is used.
        retries: Total number of retries per request.
        backoff_factor: Multiplier for retry delay times (1, 2, 4, ...)

    Returns:
        Tuple of file path, total number of uploaded bytes.

    Raises:
        openapi_client.exceptions.ApiException: raised on unexpected server
            responses (HTTP status codes other than 206, 413, 429, 503)
        aiohttp.ClientError: various low-level HTTP errors that persist
            after retrying: connection errors, timeouts, decode errors,
            invalid HTTP headers, payload too large (HTTP 413), too many
            requests (HTTP 429), service unavailable (HTTP 503)
    """
    # cast string to Path if necessary
    if isinstance(target, str):
        target = Path(target)

    # if it is a directory, take the filename from the url
    if target.is_dir():
        target = target / urlparse(url)[2].rsplit("/", 1)[-1]

    # open the file
    try:
        async with aiofiles.open(target, "wb", executor=executor) as fileobj:
            size = await download_fileobj(
                url,
                fileobj,
                chunk_size=chunk_size,
                timeout=timeout,
                connector=connector,
                retries=retries,
                backoff_factor=backoff_factor,
            )
    except Exception:
        # Clean up a partially downloaded file
        try:
            await aiofiles.os.remove(target)
        except FileNotFoundError:
            pass
        raise

    return target, size


async def _download_request(client, start, stop, url, timeout, retries, backoff_factor):
    """Send a download with a byte range & parse the content-range header"""
    headers = {"Range": "bytes={}-{}".format(start, stop - 1)}
    request = partial(
        client.request,
        "GET",
        url,
        headers=headers,
        timeout=timeout,
    )
    logger.debug("Downloading bytes {} to {}...".format(start, stop))
    response = await _request_with_retry(request, retries, backoff_factor)
    logger.debug("Finished downloading bytes {} to {}".format(start, stop))
    if response.status == 200:
        raise ApiException(
            status=200,
            reason="The file server does not support multipart downloads.",
        )
    elif response.status != 206:
        raise ApiException(http_resp=response)
    # parse content-range header (e.g. "bytes 0-3/7") for next iteration
    content_range = response.headers["Content-Range"]
    start, stop, total = [
        int(x) for x in CONTENT_RANGE_REGEXP.findall(content_range)[0]
    ]
    return response, total


async def download_fileobj(
    url: str,
    fileobj,
    chunk_size: int = 16777216,
    timeout: float = 300.0,
    connector: Optional[aiohttp.BaseConnector] = None,
    retries: int = 3,
    backoff_factor: float = 1.0,
) -> int:
    """Download a url to a file object using multiple requests.

    It is assumed that the file server supports multipart downloads (range
    requests).

    Args:
        url: The url to retrieve.
        fileobj: The (binary) file object to write into, supporting async I/O.
        chunk_size: The number of bytes per request. Default: 16MB.
        timeout: The timeout of the download of a single chunk in seconds.
        connector: An optional aiohttp connector to support connection pooling.
            If not supplied, a default TCPConnector is instantiated with a pool
            size (limit) of 4.
        retries: Total number of retries per request.
        backoff_factor: Multiplier for retry delay times (1, 2, 4, ...)

    Returns:
        The total number of downloaded bytes.

    Raises:
        openapi_client.exceptions.ApiException: raised on unexpected server
            responses (HTTP status codes other than 206, 413, 429, 503)
        aiohttp.ClientError: various low-level HTTP errors that persist
            after retrying: connection errors, timeouts, decode errors,
            invalid HTTP headers, payload too large (HTTP 413), too many
            requests (HTTP 429), service unavailable (HTTP 503)

        Note that the fileobj might be partially filled with data in case of
        an exception.
    """
    if connector is None:
        connector = aiohttp.TCPConnector(limit=DEFAULT_CONN_LIMIT)

    # Our strategy here is to download the first chunk, get the total file
    # size from the header, and then parrellelize the rest of the chunks.
    # We could get the total Content-Length from a HEAD request, but not all
    # servers support that (e.g. Minio).
    request_kwargs = {
        "url": url,
        "timeout": timeout,
        "retries": retries,
        "backoff_factor": backoff_factor,
    }
    async with aiohttp.ClientSession(connector=connector) as client:
        # start with a single chunk to learn the total file size
        response, file_size = await _download_request(
            client, 0, chunk_size, **request_kwargs
        )

        # write to file
        await fileobj.write(await response.read())
        logger.debug("Written bytes {} to {} to file".format(0, chunk_size))

        # return if the file is already completely downloaded
        if file_size <= chunk_size:
            return file_size

        # create tasks for the rest of the chunks
        tasks = [
            asyncio.ensure_future(
                _download_request(client, start, start + chunk_size, **request_kwargs)
            )
            for start in range(chunk_size, file_size, chunk_size)
        ]

        # write the result of the tasks to the file one by one
        try:
            for i, task in enumerate(tasks, 1):
                response, _ = await task
                # write to file
                await fileobj.write(await response.read())
                logger.debug(
                    "Written bytes {} to {} to file".format(
                        i * chunk_size, (i + 1) * chunk_size
                    )
                )
        except Exception:
            # in case of an exception, cancel all tasks
            for task in tasks:
                task.cancel()
            raise

        return file_size


async def upload_file(
    url: str,
    file_path: Path,
    chunk_size: int = 16777216,
    timeout: float = 300.0,
    connector: Optional[aiohttp.BaseConnector] = None,
    md5: Optional[bytes] = None,
    executor: Optional[ThreadPoolExecutor] = None,
    retries: int = 3,
    backoff_factor: float = 1.0,
) -> int:
    """Upload a file at specified file path to a url.

    The upload is accompanied by an MD5 hash so that the file server checks
    the integrity of the file.

    Args:
        url: The url to upload to.
        file_path: The file path to read data from.
        chunk_size: The size of the chunk in the streaming upload. Note that this
            function does not do multipart upload. Default: 16MB.
        timeout: The total timeout of the upload in seconds.
        connector: An optional aiohttp connector to support connection pooling.
        md5: The MD5 digest (binary) of the file. Supply the MD5 if you already
            have access to it. Otherwise this function will compute it for you.
        executor: The ThreadPoolExecutor to execute local file I/O and MD5 hashing
            in. If not supplied, default executor is used.
        retries: Total number of retries per request.
        backoff_factor: Multiplier for retry delay times (1, 2, 4, ...)

    Returns:
        The total number of uploaded bytes.

    Raises:
        IOError: Raised if the provided file is incompatible or empty.
        openapi_client.exceptions.ApiException: raised on unexpected server
            responses (HTTP status codes other than 206, 413, 429, 503)
        aiohttp.ClientError: various low-level HTTP errors that persist
            after retrying: connection errors, timeouts, decode errors,
            invalid HTTP headers, payload too large (HTTP 413), too many
            requests (HTTP 429), service unavailable (HTTP 503)
    """
    # cast string to Path if necessary
    if isinstance(file_path, str):
        file_path = Path(file_path)

    # open the file
    async with aiofiles.open(file_path, "rb", executor=executor) as fileobj:
        size = await upload_fileobj(
            url,
            fileobj,
            chunk_size=chunk_size,
            timeout=timeout,
            connector=connector,
            md5=md5,
            executor=executor,
            retries=retries,
            backoff_factor=backoff_factor,
        )

    return size


async def _iter_chunks(fileobj, chunk_size: int):
    """Yield chunks from a file stream"""
    assert chunk_size > 0
    while True:
        data = await fileobj.read(chunk_size)
        if len(data) == 0:
            break
        yield data


async def _compute_md5(
    fileobj,
    chunk_size: int,
    executor: Optional[ThreadPoolExecutor] = None,
):
    """Return the md5 digest for given fileobj."""
    loop = asyncio.get_event_loop()

    await fileobj.seek(0)
    hasher = hashlib.md5()
    async for chunk in _iter_chunks(fileobj, chunk_size=chunk_size):
        # From python docs: the Python GIL is released for data larger than
        # 2047 bytes at object creation or on update.
        # So it makes sense to do the hasher updates in a threadpool.
        await loop.run_in_executor(executor, partial(hasher.update, chunk))
    return await loop.run_in_executor(executor, hasher.digest)


async def _upload_request(client, fileobj, chunk_size, *args, **kwargs):
    """Send a request with the contents of fileobj as iterable in the body"""
    await fileobj.seek(0)
    return await client.request(
        *args,
        data=_iter_chunks(fileobj, chunk_size=chunk_size),
        **kwargs,
    )


async def upload_fileobj(
    url: str,
    fileobj,
    chunk_size: int = 16777216,
    timeout: float = 300.0,
    connector: Optional[aiohttp.BaseConnector] = None,
    md5: Optional[bytes] = None,
    executor: Optional[ThreadPoolExecutor] = None,
    retries: int = 3,
    backoff_factor: float = 1.0,
) -> int:
    """Upload a file object to a url.

    The upload is accompanied by an MD5 hash so that the file server checks
    the integrity of the file.

    Args:
        url: The url to upload to.
        fileobj: The (binary) file object to read from, supporting async I/O.
        chunk_size: The size of the chunk in the streaming upload. Note that this
            function does not do multipart upload. Default: 16MB.
        timeout: The total timeout of the upload in seconds.
        connector: An optional aiohttp connector to support connection pooling.
        md5: The MD5 digest (binary) of the file. Supply the MD5 if you already
            have access to it. Otherwise this function will compute it for you.
        executor: The ThreadPoolExecutor to execute MD5 hashing in. If not
            supplied, default executor is used.
        retries: Total number of retries per request.
        backoff_factor: Multiplier for retry delay times (1, 2, 4, ...)

    Returns:
        The total number of uploaded bytes.

    Raises:
        IOError: Raised if the provided file is incompatible or empty.
        openapi_client.exceptions.ApiException: raised on unexpected server
            responses (HTTP status codes other than 206, 413, 429, 503)
        aiohttp.ClientError: various low-level HTTP errors that persist
            after retrying: connection errors, timeouts, decode errors,
            invalid HTTP headers, payload too large (HTTP 413), too many
            requests (HTTP 429), service unavailable (HTTP 503)
    """
    # There are two ways to upload in S3 (Minio):
    # - PutObject: put the whole object in one time
    # - multipart upload: requires presigned urls for every part
    # We can only do the first option as we have no other presigned urls.
    # So we take the first option, but we do stream the request body in chunks.

    # We will get hard to understand tracebacks if the fileobj is not
    # in binary mode. So use a trick to see if fileobj is in binary mode:
    if not isinstance(await fileobj.read(0), bytes):
        raise IOError(
            "The file object is not in binary mode. Please open with mode='rb'."
        )

    # For computing the MD5 we need to do an extra pass on the file.
    if md5 is None:
        md5 = await _compute_md5(fileobj, chunk_size, executor=executor)

    file_size = await fileobj.seek(0, 2)  # go to EOF to get the file size
    if file_size == 0:
        raise IOError("The file object is empty.")

    # Tested: both Content-Length and Content-MD5 are checked by Minio
    headers = {
        "Content-Length": str(file_size),
        "Content-MD5": base64.b64encode(md5).decode(),
    }

    async with aiohttp.ClientSession(connector=connector) as client:
        request = partial(
            _upload_request,
            client,
            fileobj,
            chunk_size,
            "PUT",
            url,
            headers=headers,
            timeout=timeout,
        )
        response = await _request_with_retry(request, retries, backoff_factor)
        if response.status != 200:
            raise ApiException(status=response.status, reason=response.reason)

    return file_size



from __future__ import absolute_import

# flake8: noqa

# import apis into api package
from openapi_client.api.auth_api import AuthApi
from openapi_client.api.contracts_api import ContractsApi
from openapi_client.api.files_api import FilesApi
from openapi_client.api.inpy_versions_api import InpyVersionsApi
from openapi_client.api.organisations_api import OrganisationsApi
from openapi_client.api.permissions_api import PermissionsApi
from openapi_client.api.repositories_api import RepositoriesApi
from openapi_client.api.revisions_api import RevisionsApi
from openapi_client.api.roles_api import RolesApi
from openapi_client.api.simulations_api import SimulationsApi
from openapi_client.api.statuses_api import StatusesApi
from openapi_client.api.threedimodels_api import ThreedimodelsApi
from openapi_client.api.usage_api import UsageApi



# coding: utf-8

"""
    3Di API

    3Di simulation API (latest version: 3.0)   Framework release: 1.0.16   3Di core release: 2.0.11  deployed on:  07:33AM (UTC) on September 04, 2020  # noqa: E501

    The version of the OpenAPI document: 3.0
    Contact: info@nelen-schuurmans.nl
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class BasicPostProcessing(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'simulation': 'str',
        'scenario_name': 'str',
        'process_basic_results': 'bool',
        'result_uuid': 'str'
    }

    attribute_map = {
        'simulation': 'simulation',
        'scenario_name': 'scenario_name',
        'process_basic_results': 'process_basic_results',
        'result_uuid': 'result_uuid'
    }

    def __init__(self, simulation=None, scenario_name=None, process_basic_results=None, result_uuid=None, local_vars_configuration=None):  # noqa: E501
        """BasicPostProcessing - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._simulation = None
        self._scenario_name = None
        self._process_basic_results = None
        self._result_uuid = None
        self.discriminator = None

        if simulation is not None:
            self.simulation = simulation
        if scenario_name is not None:
            self.scenario_name = scenario_name
        if process_basic_results is not None:
            self.process_basic_results = process_basic_results
        if result_uuid is not None:
            self.result_uuid = result_uuid

    @property
    def simulation(self):
        """Gets the simulation of this BasicPostProcessing.  # noqa: E501


        :return: The simulation of this BasicPostProcessing.  # noqa: E501
        :rtype: str
        """
        return self._simulation

    @simulation.setter
    def simulation(self, simulation):
        """Sets the simulation of this BasicPostProcessing.


        :param simulation: The simulation of this BasicPostProcessing.  # noqa: E501
        :type: str
        """

        self._simulation = simulation

    @property
    def scenario_name(self):
        """Gets the scenario_name of this BasicPostProcessing.  # noqa: E501

        Scenario name for saving the results  # noqa: E501

        :return: The scenario_name of this BasicPostProcessing.  # noqa: E501
        :rtype: str
        """
        return self._scenario_name

    @scenario_name.setter
    def scenario_name(self, scenario_name):
        """Sets the scenario_name of this BasicPostProcessing.

        Scenario name for saving the results  # noqa: E501

        :param scenario_name: The scenario_name of this BasicPostProcessing.  # noqa: E501
        :type: str
        """
        if (self.local_vars_configuration.client_side_validation and
                scenario_name is not None and len(scenario_name) > 128):
            raise ValueError("Invalid value for `scenario_name`, length must be less than or equal to `128`")  # noqa: E501
        if (self.local_vars_configuration.client_side_validation and
                scenario_name is not None and len(scenario_name) < 1):
            raise ValueError("Invalid value for `scenario_name`, length must be greater than or equal to `1`")  # noqa: E501

        self._scenario_name = scenario_name

    @property
    def process_basic_results(self):
        """Gets the process_basic_results of this BasicPostProcessing.  # noqa: E501


        :return: The process_basic_results of this BasicPostProcessing.  # noqa: E501
        :rtype: bool
        """
        return self._process_basic_results

    @process_basic_results.setter
    def process_basic_results(self, process_basic_results):
        """Sets the process_basic_results of this BasicPostProcessing.


        :param process_basic_results: The process_basic_results of this BasicPostProcessing.  # noqa: E501
        :type: bool
        """

        self._process_basic_results = process_basic_results

    @property
    def result_uuid(self):
        """Gets the result_uuid of this BasicPostProcessing.  # noqa: E501


        :return: The result_uuid of this BasicPostProcessing.  # noqa: E501
        :rtype: str
        """
        return self._result_uuid

    @result_uuid.setter
    def result_uuid(self, result_uuid):
        """Sets the result_uuid of this BasicPostProcessing.


        :param result_uuid: The result_uuid of this BasicPostProcessing.  # noqa: E501
        :type: str
        """

        self._result_uuid = result_uuid

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, BasicPostProcessing):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, BasicPostProcessing):
            return True

        return self.to_dict() != other.to_dict()



import io
from concurrent.futures import ThreadPoolExecutor

import pytest
from aiofiles.threadpool import AsyncBufferedIOBase

# note: unittest.mock has no asyncio support in Python < 3.7,
# but luckily mock backported it:
from mock import AsyncMock, Mock, patch

from openapi_client.exceptions import ApiException
from threedi_api_client.aio.files import (
    download_file,
    download_fileobj,
    upload_file,
    upload_fileobj,
)


class AsyncBytesIO:
    """Just for testing, dumb async version of BytesIO"""

    def __init__(self):
        self._io = io.BytesIO()

    async def tell(self, *args, **kwargs):
        return self._io.tell(*args, **kwargs)

    async def seek(self, *args, **kwargs):
        return self._io.seek(*args, **kwargs)

    async def read(self, *args, **kwargs):
        return self._io.read(*args, **kwargs)

    async def write(self, *args, **kwargs):
        return self._io.write(*args, **kwargs)


@pytest.fixture
async def aio_request():
    with patch(
        "aiohttp.ClientSession.request", new_callable=AsyncMock
    ) as aio_request:
        yield aio_request


@pytest.fixture
async def response_error():
    # mimics aiohttp.ClientResponse
    response = AsyncMock()
    response.status = 503
    return response


@pytest.fixture
async def responses_single():
    # mimics aiohttp.ClientResponse
    response = AsyncMock()
    response.headers = {"Content-Range": "bytes 0-41/42"}
    response.status = 206
    response.read = AsyncMock(return_value=b"X" * 42)
    return [response]


@pytest.fixture
async def responses_double():
    # mimics aiohttp.ClientResponse
    response1 = AsyncMock()
    response1.headers = {"Content-Range": "bytes 0-63/65"}
    response1.status = 206
    response1.read = AsyncMock(return_value=b"X" * 64)
    response2 = AsyncMock()
    response2.headers = {"Content-Range": "bytes 64-64/65"}
    response2.status = 206
    response2.read = AsyncMock(return_value=b"X")
    return [response1, response2]


@pytest.mark.asyncio
async def test_download_fileobj(aio_request, responses_single):
    stream = AsyncBytesIO()
    aio_request.side_effect = responses_single

    await download_fileobj("some-url", stream, chunk_size=64)

    aio_request.assert_called_with(
        "GET",
        "some-url",
        headers={"Range": "bytes=0-63"},
        timeout=300.0,
    )
    assert await stream.tell() == 42


@pytest.mark.asyncio
async def test_download_fileobj_two_chunks(aio_request, responses_double):
    stream = AsyncBytesIO()
    aio_request.side_effect = responses_double

    await download_fileobj("some-url", stream, chunk_size=64)

    (_, kwargs1), (_, kwargs2) = aio_request.call_args_list
    assert kwargs1["headers"] == {"Range": "bytes=0-63"}
    assert kwargs2["headers"] == {"Range": "bytes=64-127"}
    assert await stream.tell() == 65


@pytest.mark.asyncio
@patch("asyncio.sleep", new_callable=AsyncMock)
async def test_download_fileobj_retry(
    sleep, aio_request, responses_double, response_error
):
    stream = AsyncBytesIO()
    responses = [
        response_error,
        responses_double[0],
        response_error,
        response_error,
        responses_double[1],
    ]
    aio_request.side_effect = responses

    await download_fileobj(
        "some-url", stream, chunk_size=64, retries=3, backoff_factor=1.5
    )

    assert aio_request.call_count == 5
    assert sleep.call_count == 3
    assert [x[0][0] for x in sleep.call_args_list] == [1.5, 1.5, 3.0]
    assert await stream.tell() == 65


@pytest.mark.asyncio
@patch("asyncio.sleep", new_callable=AsyncMock)
async def test_download_fileobj_retry_limit(sleep, aio_request, response_error):
    aio_request.side_effect = [response_error] * 2

    with pytest.raises(ApiException) as e:
        await download_fileobj("some-url", None, chunk_size=64, retries=2)

    assert e.value.status == response_error.status
    assert aio_request.call_count == 2


@pytest.mark.asyncio
async def test_download_fileobj_no_multipart(aio_request, responses_single):
    """The remote server does not support range requests"""
    responses_single[0].status = 200
    aio_request.side_effect = responses_single

    with pytest.raises(ApiException) as e:
        await download_fileobj("some-url", None, chunk_size=64)

    assert e.value.status == 200
    assert e.value.reason == "The file server does not support multipart downloads."


@pytest.mark.asyncio
async def test_download_fileobj_forbidden_2nd_chunk(aio_request, responses_double):
    """The remote server does not support range requests"""
    stream = AsyncBytesIO()
    responses_double[1].status = 403
    aio_request.side_effect = responses_double

    with pytest.raises(ApiException) as e:
        await download_fileobj("some-url", stream, chunk_size=64)

    assert e.value.status == 403


@pytest.mark.asyncio
async def test_download_fileobj_forbidden(aio_request, response_error):
    """The remote server does not support range requests"""
    response_error.status = 403
    aio_request.side_effect = [response_error]

    with pytest.raises(ApiException) as e:
        await download_fileobj("some-url", None, chunk_size=64)

    assert e.value.status == 403


@pytest.mark.asyncio
@patch("threedi_api_client.aio.files.download_fileobj", new_callable=AsyncMock)
async def test_download_file(mocked_download_fileobj, tmp_path):
    executor = ThreadPoolExecutor()

    await download_file(
        "http://domain/a.b",
        tmp_path / "c.d",
        chunk_size=64,
        timeout=3.0,
        connector="foo",
        executor=executor,
        retries=2,
        backoff_factor=1.5,
    )

    args, kwargs = mocked_download_fileobj.call_args
    assert args[0] == "http://domain/a.b"
    assert isinstance(args[1], AsyncBufferedIOBase)
    assert args[1].mode == "wb"
    assert args[1].name == str(tmp_path / "c.d")
    assert kwargs["chunk_size"] == 64
    assert kwargs["timeout"] == 3.0
    assert kwargs["connector"] == "foo"
    assert kwargs["retries"] == 2
    assert kwargs["backoff_factor"] == 1.5
    assert "executor" not in kwargs  # download_fileobj does not expect it


@pytest.mark.asyncio
@patch("threedi_api_client.aio.files.download_fileobj", new_callable=AsyncMock)
async def test_download_file_directory(mocked_download_fileobj, tmp_path):
    # if a target directory is specified, a filename is created from the url
    await download_file(
        "http://domain/a.b", tmp_path, chunk_size=64, timeout=3.0, connector="foo"
    )

    args, kwargs = mocked_download_fileobj.call_args
    assert args[1].name == str(tmp_path / "a.b")


@pytest.fixture
async def upload_response():
    response = AsyncMock()
    response.status = 200
    return response


@pytest.fixture
async def fileobj():
    stream = AsyncBytesIO()
    await stream.write(b"X" * 39)
    await stream.seek(0)
    return stream


@pytest.mark.asyncio
@pytest.mark.parametrize(
    "chunk_size,expected_body",
    [
        (64, [b"X" * 39]),
        (39, [b"X" * 39]),
        (38, [b"X" * 38, b"X"]),
        (16, [b"X" * 16, b"X" * 16, b"X" * 7]),
    ],
)
async def test_upload_fileobj(
    aio_request, fileobj, upload_response, chunk_size, expected_body
):
    aio_request.return_value = upload_response
    await upload_fileobj("some-url", fileobj, chunk_size=chunk_size)

    # base64.b64encode(hashlib.md5(b"X" * 39).digest()).decode()
    expected_md5 = "Q2zMNJgyazDIkoSqvpOqVg=="

    args, kwargs = aio_request.call_args
    assert args == ("PUT", "some-url")
    assert [x async for x in kwargs["data"]] == expected_body
    assert kwargs["headers"] == {"Content-Length": "39", "Content-MD5": expected_md5}
    assert kwargs["timeout"] == 300.0


@pytest.mark.asyncio
async def test_upload_fileobj_with_md5(aio_request, fileobj, upload_response):
    aio_request.return_value = upload_response
    await upload_fileobj("some-url", fileobj, md5=b"abcd")

    # base64.b64encode(b"abcd")).decode()
    expected_md5 = "YWJjZA=="

    args, kwargs = aio_request.call_args
    assert kwargs["headers"] == {"Content-Length": "39", "Content-MD5": expected_md5}


@pytest.mark.asyncio
async def test_upload_fileobj_empty_file():
    with pytest.raises(IOError, match="The file object is empty."):
        await upload_fileobj("some-url", AsyncBytesIO())


@pytest.mark.asyncio
async def test_upload_fileobj_non_binary_file():
    string_io = AsyncMock()
    string_io.read = AsyncMock(return_value="some string")
    with pytest.raises(IOError, match="The file object is not in binary*"):
        await upload_fileobj("some-url", string_io)


@pytest.mark.asyncio
async def test_upload_fileobj_errors(aio_request, fileobj, upload_response):
    upload_response.status = 400
    aio_request.return_value = upload_response
    with pytest.raises(ApiException) as e:
        await upload_fileobj("some-url", fileobj)

    assert e.value.status == 400


@pytest.mark.asyncio
@patch("threedi_api_client.aio.files.upload_fileobj", new_callable=AsyncMock)
async def test_upload_file(upload_fileobj, tmp_path):
    executor = ThreadPoolExecutor()

    path = tmp_path / "myfile"
    with path.open("wb") as f:
        f.write(b"X")

    await upload_file(
        "http://domain/a.b",
        path,
        chunk_size=1234,
        timeout=3.0,
        connector="foo",
        executor=executor,
        md5=b"abcd",
        retries=2,
        backoff_factor=1.5,
    )

    args, kwargs = upload_fileobj.call_args
    assert args[0] == "http://domain/a.b"
    assert isinstance(args[1], AsyncBufferedIOBase)
    assert args[1].mode == "rb"
    assert args[1].name == str(path)
    assert kwargs["timeout"] == 3.0
    assert kwargs["chunk_size"] == 1234
    assert kwargs["md5"] == b"abcd"
    assert kwargs["connector"] == "foo"
    assert kwargs["executor"] is executor
    assert kwargs["retries"] == 2
    assert kwargs["backoff_factor"] == 1.5
